
BASH_WITH_COMMANDS = """
# in case is not defined
if [[ $(type -t _get_comp_words_by_ref) == "" ]]; then
	_get_comp_words_by_ref() {{
		local exclude flag i OPTIND=1
		local cur cword words=()
		local upargs=() upvars=() vcur vcword vprev vwords

		while getopts "c:i:n:p:w:" flag "$@"; do
			case $flag in
				c) vcur=$OPTARG ;;
				i) vcword=$OPTARG ;;
				n) exclude=$OPTARG ;;
				p) vprev=$OPTARG ;;
				w) vwords=$OPTARG ;;
			esac
		done
		while [[ $# -ge $OPTIND ]]; do
			case ${{!OPTIND}} in
				cur)   vcur=cur ;;
				prev)  vprev=prev ;;
				cword) vcword=cword ;;
				words) vwords=words ;;
				*) echo "bash_completion: $FUNCNAME: \\`${{!OPTIND}}':" \\
					"unknown argument" >&2; return 1 ;;
			esac
			(( OPTIND += 1 ))
		done

		__get_cword_at_cursor_by_ref "$exclude" words cword cur

		[[ $vcur   ]] && {{ upvars+=("$vcur"  ); upargs+=(-v $vcur   "$cur"  ); }}
		[[ $vcword ]] && {{ upvars+=("$vcword"); upargs+=(-v $vcword "$cword"); }}
		[[ $vprev && $cword -ge 1 ]] && {{ upvars+=("$vprev" ); upargs+=(-v $vprev
			"${{words[cword - 1]}}"); }}
		[[ $vwords ]] && {{ upvars+=("$vwords"); upargs+=(-a${{#words[@]}} $vwords
			"${{words[@]}}"); }}

		(( ${{#upvars[@]}} )) && local "${{upvars[@]}}" && _upvars "${{upargs[@]}}"
	}}
fi

if [[ $(type -t __ltrim_colon_completions) == "" ]]; then
	__ltrim_colon_completions() {{
		if [[ "$1" == *:* && "$COMP_WORDBREAKS" == *:* ]]; then
			# Remove colon-word prefix from COMPREPLY items
			local colon_word=${{1%"${{1##*:}}"}}
			local i=${{#COMPREPLY[*]}}
			while [[ $((--i)) -ge 0 ]]; do
				COMPREPLY[$i]=${{COMPREPLY[$i]#"$colon_word"}}
			done
		fi
	}}
fi

{complete_function}() {{
	local cur script coms opts com
	COMPREPLY=()
	_get_comp_words_by_ref -n : cur words

	# for an alias, get the real script behind it
	if [[ $(type -t ${{words[0]}}) == "alias" ]]; then
		script=$(alias ${{words[0]}} | sed -E "s/alias ${{words[0]}}='(.*)'/\\1/")
	else
		script=${{words[0]}}
	fi

	# lookup for command
	for word in ${{words[@]:1}}; do
		if [[ $word != -* ]]; then
			com=$word
			break
		fi
	done

	# completing for an option
	if [[ ${{cur}} == -* ]] ; then
		opts="{global_options}"

		case "$com" in
{command_block}
		esac

		COMPREPLY=($(compgen -W "${{opts}}" -- ${{cur}}))
		__ltrim_colon_completions "$cur"

		return 0;
	fi

	# completing for a command
	if [[ $cur == $com ]]; then
		coms="{commands}"

		COMPREPLY=($(compgen -W "${{coms}}" -- ${{cur}}))
		__ltrim_colon_completions "$cur"

		return 0
	fi
}}

{exclude_block}
"""

BASH_WITH_COMMANDS_COMMAND = """
			({command})
			opts="${{opts}} {options}"
			;;
"""

BASH_EXECUTE = "complete -o default -F {complete_function!r} {name!r}"


BASH_WITHOUT_COMMANDS = """
# in case is not defined
if [[ $(type -t _get_comp_words_by_ref) == "" ]]; then
	_get_comp_words_by_ref() {{
		local exclude flag i OPTIND=1
		local cur cword words=()
		local upargs=() upvars=() vcur vcword vprev vwords

		while getopts "c:i:n:p:w:" flag "$@"; do
			case $flag in
				c) vcur=$OPTARG ;;
				i) vcword=$OPTARG ;;
				n) exclude=$OPTARG ;;
				p) vprev=$OPTARG ;;
				w) vwords=$OPTARG ;;
			esac
		done
		while [[ $# -ge $OPTIND ]]; do
			case ${{!OPTIND}} in
				cur)   vcur=cur ;;
				prev)  vprev=prev ;;
				cword) vcword=cword ;;
				words) vwords=words ;;
				*) echo "bash_completion: $FUNCNAME: \\`${{!OPTIND}}':" \\
					"unknown argument" >&2; return 1 ;;
			esac
			(( OPTIND += 1 ))
		done

		__get_cword_at_cursor_by_ref "$exclude" words cword cur

		[[ $vcur   ]] && {{ upvars+=("$vcur"  ); upargs+=(-v $vcur   "$cur"  ); }}
		[[ $vcword ]] && {{ upvars+=("$vcword"); upargs+=(-v $vcword "$cword"); }}
		[[ $vprev && $cword -ge 1 ]] && {{ upvars+=("$vprev" ); upargs+=(-v $vprev
			"${{words[cword - 1]}}"); }}
		[[ $vwords ]] && {{ upvars+=("$vwords"); upargs+=(-a${{#words[@]}} $vwords
			"${{words[@]}}"); }}

		(( ${{#upvars[@]}} )) && local "${{upvars[@]}}" && _upvars "${{upargs[@]}}"
	}}
fi

if [[ $(type -t __ltrim_colon_completions) == "" ]]; then
	__ltrim_colon_completions() {{
		if [[ "$1" == *:* && "$COMP_WORDBREAKS" == *:* ]]; then
			# Remove colon-word prefix from COMPREPLY items
			local colon_word=${{1%"${{1##*:}}"}}
			local i=${{#COMPREPLY[*]}}
			while [[ $((--i)) -ge 0 ]]; do
				COMPREPLY[$i]=${{COMPREPLY[$i]#"$colon_word"}}
			done
		fi
	}}
fi

{complete_function}() {{
	local cur script coms opts com
	COMPREPLY=()
	_get_comp_words_by_ref -n : cur words

	# for an alias, get the real script behind it
	if [[ $(type -t ${{words[0]}}) == "alias" ]]; then
		script=$(alias ${{words[0]}} | sed -E "s/alias ${{words[0]}}='(.*)'/\\1/")
	else
		script=${{words[0]}}
	fi

	# completing for an option
	if [[ ${{cur}} == -* ]] ; then
		opts="{options}"

		COMPREPLY=($(compgen -W "${{opts}}" -- ${{cur}}))
		__ltrim_colon_completions "$cur"

		return 0;
	fi
}}

{exclude_block}
"""

FISH_WITH_COMMANDS = """
function {no_command_function}
		for i in (commandline -opc)
				if contains -- $i self generate
						return 1
				end
		end
		return 0
end

# global options
{global_option_block}

# commands
{command_block}

# command options
{command_option_block}
"""

FISH_WITH_COMMANDS_GLOBAL_SHORT_OPTION = \
	"complete -c {name!r} -n {no_command_function!r} -s {option!r} -d {desc!r}"
FISH_WITH_COMMANDS_GLOBAL_LONG_OPTION = \
	"complete -c {name!r} -n {no_command_function!r} -l {option!r} -d {desc!r}"
FISH_WITH_COMMANDS_GLOBAL_OLD_OPTION = \
	"complete -c {name!r} -n {no_command_function!r} -o {option!r} -d {desc!r}"
FISH_WITH_COMMANDS_COMMAND = \
	"complete -c {name!r} -f -n {no_command_function!r} -a {command!r} -d {desc!r}"
FISH_WITH_COMMANDS_COMMAND_SHORT_OPTION = \
	"complete -c {name!r} -A -n '__fish_seen_subcommand_from {command}' -s {option!r} -d {desc!r}"
FISH_WITH_COMMANDS_COMMAND_LONG_OPTION = \
	"complete -c {name!r} -A -n '__fish_seen_subcommand_from {command}' -l {option!r} -d {desc!r}"
FISH_WITH_COMMANDS_COMMAND_OLD_OPTION = \
	"complete -c {name!r} -A -n '__fish_seen_subcommand_from {command}' -o {option!r} -d {desc!r}"

FISH_WITHOUT_COMMANDS = """
{option_block}
"""

FISH_WITHOUT_COMMANDS_SHORT_OPTION = "complete -c {name!r} -s {option!r} -d {desc!r}"
FISH_WITHOUT_COMMANDS_LONG_OPTION = "complete -c {name!r} -l {option!r} -d {desc!r}"
FISH_WITHOUT_COMMANDS_OLD_OPTION = "complete -c {name!r} -o {option!r} -d {desc!r}"

ZSH_WITH_COMMANDS = """#compdef {name}

{complete_function}() {{
	local state com cur

	cur=${{words[${{#words[@]}}]}}

	# lookup for command
	for word in ${{words[@]:1}}; do
		if [[ $word != -* ]]; then
			com=$word
			break
		fi
	done

	if [[ ${{cur}} == -* ]]; then
		state="option"
		opts=({global_options})
	elif [[ $cur == $com ]]; then
		state="command"
		coms=({commands})
	fi

	case $state in
		(command)
			_describe 'command' coms
		;;
		(option)
			case "$com" in
{command_block}
			esac

			_describe 'option' opts
		;;
		*)
			# fallback to file completion
			_arguments '*:file:_files'
	esac
}}

{complete_function} "$@"
compdef {complete_function} {fullpath}
"""

ZSH_WITHOUT_COMMANDS = """#compdef {name}

{complete_function}() {{
	local cur

	cur=${{words[${{#words[@]}}]}}

	if [[ ${{cur}} == -* ]]; then
		state="option"
		opts=({options})
		_describe 'option' opts
	else
		# fallback to file completion
		_arguments '*:file:_files'
	fi
}}

{complete_function} "$@"
compdef {complete_function} {fullpath}
"""

ZSH_WITH_COMMANDS_COMMAND = """
			({command})
			opts=({options})
			;;
"""

def _optionStyle(option):
	"""
	Tell the style of an option, when it's short '-a' or long '--apt'
	or old long '-opt' or a command 'show'
	"""
	if option.startswith('--'):
		return 'long'
	if not option.startswith('-'):
		return 'commnad'
	if len(option) > 2:
		return 'oldlong'
	return 'short'

def assembleBashWithCommands(name, complete_function, global_options, commands, fullpath = None):
	command_block = [
		BASH_WITH_COMMANDS_COMMAND.format(
			command = command.name,
			options = ' '.join(command.options.keys())
		) for command in commands.values()
	]
	exclude_block = [BASH_EXECUTE.format(complete_function = complete_function, name = name)]
	if fullpath:
		exclude_block.append([
			BASH_EXECUTE.format(complete_function = complete_function, name = fullpath)])

	return BASH_WITH_COMMANDS.format(
		complete_function = complete_function,
		global_options    = ' '.join(global_options.keys()),
		command_block     = '\n'.join(command_block),
		commands          = ' '.join(commands.keys()),
		exclude_block     = '\n'.join(exclude_block)
	)

def assembleBashWithoutCommands(name, complete_function, options, fullpath = None):
	exclude_block = [BASH_EXECUTE.format(complete_function = complete_function, name = name)]
	if fullpath:
		exclude_block.append([
			BASH_EXECUTE.format(complete_function = complete_function, name = fullpath)])
	return BASH_WITHOUT_COMMANDS.format(
		complete_function = complete_function,
		options           = ' '.join(options.keys()),
		exclude_block     = '\n'.join(exclude_block)
	)

def assembleFishWithCommands(name, no_command_function, global_options, commands, fullpath = None):
	global_option_block = [
		(     FISH_WITH_COMMANDS_GLOBAL_LONG_OPTION  if _optionStyle(option) == 'long'  \
		 else FISH_WITH_COMMANDS_GLOBAL_SHORT_OPTION if _optionStyle(option) == 'short' \
		 else FISH_WITH_COMMANDS_GLOBAL_OLD_OPTION).format(
			name                = name,
			no_command_function = no_command_function,
			option              = option.lstrip('-'),
			desc                = desc
		) for option, desc in global_options.items()
	]
	command_block = [
		FISH_WITH_COMMANDS_COMMAND.format(
			name                = name,
			no_command_function = no_command_function,
			command             = comname,
			desc                = command.desc
		) for comname, command in commands.items()
	]
	command_option_block = [
		(     FISH_WITH_COMMANDS_COMMAND_LONG_OPTION  if _optionStyle(option) == 'long'  \
		 else FISH_WITH_COMMANDS_COMMAND_SHORT_OPTION if _optionStyle(option) == 'short' \
		 else FISH_WITH_COMMANDS_COMMAND_OLD_OPTION).format(
			name    = name,
			command = cmdname,
			option  = option.lstrip('-'),
			desc    = desc
		)
		for cmdname, command in commands.items()
		for option, desc in command.options.items()
	]
	return FISH_WITH_COMMANDS.format(
		no_command_function  = no_command_function,
		global_option_block  = '\n'.join(global_option_block),
		command_block        = '\n'.join(command_block),
		command_option_block = '\n'.join(command_option_block),
	)

def assembleFishWithoutCommands(name, no_command_function, options, fullpath = None):
	option_block = [
		(     FISH_WITHOUT_COMMANDS_LONG_OPTION  if _optionStyle(option) == 'long'  \
		 else FISH_WITHOUT_COMMANDS_SHORT_OPTION if _optionStyle(option) == 'short' \
		 else FISH_WITHOUT_COMMANDS_OLD_OPTION).format(
			name   = name,
			option = option.lstrip('-'),
			desc   = desc
		)
		for option, desc in options.items()
	]
	option_block.insert(0, '# ' + no_command_function)
	return FISH_WITHOUT_COMMANDS.format(
		option_block = '\n'.join(option_block)
	)

def _escapeColon(name):
	"""Escape colon in command/option name for zsh"""
	return name.replace(':', '\\:')

def assembleZshWithCommands(name, complete_function, global_options, commands, fullpath = None):
	command_block = [
		ZSH_WITH_COMMANDS_COMMAND.format(
			command = comname,
			options = ' '.join(repr(_escapeColon(option) + ':' + _escapeColon(desc))
			for option, desc in command.options.items())
		) for comname, command in commands.items()
	]
	return ZSH_WITH_COMMANDS.format(
		name              = name,
		fullpath          = fullpath,
		complete_function = complete_function,
		command_block     = '\n'.join(command_block),
		global_options    = ' '.join(repr(_escapeColon(option) + ':' + _escapeColon(desc))
			for option, desc in global_options.items()),
		commands          = ' '.join(repr(_escapeColon(comname) + ':' + _escapeColon(command.desc))
			for comname, command in commands.items()),
	)

def assembleZshWithoutCommands(name, complete_function, options, fullpath = None):
	return ZSH_WITHOUT_COMMANDS.format(
		name              = name,
		fullpath          = fullpath,
		complete_function = complete_function,
		options           = ' '.join("'%s'" % (_escapeColon(option) + ':' + _escapeColon(desc))
			for option, desc in options.items())
	)



#!/usr/bin/env python
"""
Generate completions for shells
"""
import re
import sys
import uuid
import warnings
from os import path, sep, rename, makedirs, environ
from simpleconf import Config
from completions.templates import assembleBashWithCommands, assembleBashWithoutCommands, \
	assembleFishWithCommands, assembleFishWithoutCommands, \
	assembleZshWithCommands, assembleZshWithoutCommands


def checkOptname(optname):
	"""Send warning if necessary"""
	if optname.startswith('--') and len(optname) <= 3:
		warnings.warn('Long option %r specified, but the name has length < 2' % optname)
	if optname.startswith('-') and not optname.startswith('--') and len(optname) > 2:
		warnings.warn('Short option %r specified, but the name has length > 1' % optname)

def log(msg, *args):
	"""simple log on the screen"""
	sys.stderr.write('- %s\n' % (msg % args))

class CompletionsLoadError(Exception):
	"""Raises while failed to load completions from configuration file"""

class Command:
	"""A command"""
	def __init__(self, name, desc, options = None):
		self.name = name
		self.desc = desc
		self.options = options or {}

	def addOption(self, opt, desc):
		"""
		Add option to a command
		"""
		if isinstance(opt, list):
			for option in opt:
				self.options[option] = desc
		else:
			self.options[opt] = desc

class Completions(Command):
	"""Completions class"""
	def __init__(self, name = None, desc = None, options = None, inherit = True, fullpath = None):
		super(Completions, self).__init__(name, desc, options)
		self.name     = name or sys.argv[0]
		self.desc     = desc or ''
		self.inherit  = True
		self.commands = {}
		self.inherit  = inherit
		self.uid      = None
		self.fullpath = fullpath and path.realpath(fullpath)

		if sep in self.name:
			self.fullpath = self.fullpath or path.realpath(self.name)
			self.name = path.basename(self.name)

	@property
	def availname(self):
		"""Make an available for function name"""
		return re.sub(r'[^\w_]+', '_', path.basename(self.name))

	def addCommand(self, name, desc, options = None):
		"""Add a command to the completions"""
		self.commands[name] = Command(name, desc, options)

	def command(self, name):
		"""Get the command object by given name"""
		return self.commands[name]

	def _automateFish(self, source):
		compfile = path.expanduser('~/.config/fish/completions/%s.fish' % self.name)
		backfile = compfile + '.completions.bak'
		if path.isfile(compfile):
			log('Completion file exists: %r', compfile)
			log('Back it up to: %r', backfile)
			rename(compfile, backfile)
		log('Writing completion code to: %r', compfile)
		with open(compfile, 'w') as fcomp:
			fcomp.write(source)
		log('Done, you may need to restart your shell in order for the changes to take effect.')

	def _automateBash(self, source):
		compfile = path.expanduser('~/.bash_completion.d/%s.bash-completion' % self.name)
		compdir  = path.dirname(compfile)
		backfile = compfile + '.completions.bak'
		if not path.isdir(compdir):
			log('User completion directory does not exist.')
			log('Try to create it: %s' % compdir)
			makedirs(compdir)
		entryfile = path.expanduser('~/.bashrc')
		entrybak  = entryfile + '.completions.bak'
		with open(entryfile, 'r') as fentry:
			entry = fentry.read()
		# detect if we've already add entry point
		entry_point = '\n' + \
					  '### Start adding entry point by completions, do NOT modify ###\n' + \
					  'for bcfile in %s/*.bash-completion; do\n' % compdir + \
					  '	[ -f "$bcfile" ] && . $bcfile\n' + \
					  'done\n' + \
					  '### End adding entry point by completions ###\n'

		if entry_point not in entry:
			log('Backup entry point file: %s' % entryfile)
			log('To: %s' % entrybak)
			with open(entryfile, 'a+') as fentry, open(entrybak, 'w') as fbak:
				fbak.write(fentry.read())
				log('Add entry point')
				fentry.write(entry_point)

		if path.isfile(compfile):
			log('Completion file exists: %r', compfile)
			log('Back it up to: %r', backfile)
			rename(compfile, backfile)
		log('Writing completion code to: %r', compfile)
		with open(compfile, 'w') as fcomp:
			fcomp.write(source)
		log('Done, you may need to restart your shell in order for the changes to take effect.')


	def _automateZsh(self, source):
		compfile = path.expanduser('~/.zsh-completions/_%s' % self.name)
		compdir  = path.dirname(compfile)
		backfile = path.expanduser('~/.zsh-completions/.%s.completions.bak' % self.name)
		if not path.isdir(compdir):
			log('User completion directory does not exist.')
			log('Try to create it: %s' % compdir)
			makedirs(compdir, mode = 0o755)
		entryfile = path.expanduser('~/.zshrc')
		entrybak  = entryfile + '.completions.bak'
		with open(entryfile, 'r') as fentry:
			entry = fentry.read()
		# detect if we've already add entry point

		if 'compinit' in entry:
			entry_point = '\n' + \
				'### Start adding entry point by completions, do NOT modify ###\n' + \
				'fpath+=%s\n' % compdir + \
				'### End adding entry point by completions ###\n'
		else:
			log('compinstall not found in %s' % entryfile)
			log('Add it (to disable: add `#cominit`).')
			entry_point = '\n' + \
				'### Start adding entry point by completions ###\n' + \
				'zstyle :compinstall filename %r\n' % entryfile + \
				'autoload -Uz compinit\n' + \
				'fpath+=%s\n' % compdir + \
				'# blow may take some time to start, you may want to comment it out\n' + \
				'# and set up the compinit by yourself.\n' + \
				'rm -f ~/.zcompdump; compinit -C\n' + \
				'### End adding entry point by completions ###\n'

		if '# Start adding entry point by completions' not in entry:
			log('Backup entry point file: %s' % entryfile)
			log('To: %s' % entrybak)
			with open(entryfile, 'r') as fentry, open(entrybak, 'w') as fbak:
				entrysrc = fentry.read()
				fbak.write(entrysrc)
			log('Add entry point')

			compinit_index = None
			entrylines = entrysrc.splitlines()
			for i, line in enumerate(reversed(entrylines)):
				if 'cominit' in line and not line.startswith('#'):
					compinit_index = i
					break
			if compinit_index is None:
				entrysrc += entry_point
			else:
				entrylines.insert(compinit_index, entry_point)
				entrysrc = '\n'.join(entrylines)
			with open(entryfile, 'w') as fentry:
				fentry.write(entrysrc)

		if path.isfile(compfile):
			log('Completion file exists: %r', compfile)
			log('Back it up to: %r', backfile)
			rename(compfile, backfile)
		log('Writing completion code to: %r', compfile)
		with open(compfile, 'w') as fcomp:
			fcomp.write(source)
		log('Done, you may need to restart your shell in order for the changes to take effect.')

	def _generateBash(self):
		if self.commands:
			return assembleBashWithCommands(self.name,
				'_%s_%s_complete' % (self.availname, self.uid),
				self.options, self.commands, self.fullpath)
		return assembleBashWithoutCommands(self.name,
			'_%s_%s_complete' % (self.availname, self.uid),
			self.options, self.fullpath)

	def _generateFish(self):
		if self.commands:
			return assembleFishWithCommands(self.name,
				'_%s_%s_complete' % (self.availname, self.uid),
				self.options, self.commands, self.fullpath)
		return assembleFishWithoutCommands(self.name,
			'_%s_%s_complete' % (self.availname, self.uid),
			self.options, self.fullpath)

	def _generateZsh(self):
		if self.commands:
			return assembleZshWithCommands(self.name,
				'_%s_%s_complete' % (self.availname, self.uid),
				self.options, self.commands, self.fullpath)
		return assembleZshWithoutCommands(self.name,
			'_%s_%s_complete' % (self.availname, self.uid),
			self.options, self.fullpath)

	def _inherit(self):
		if not self.inherit:
			return
		for _, command in self.commands.items():
			command.options.update(self.options)

	def generate(self, shell, auto = False):
		"""Generate the completion code"""
		self.uid = str(uuid.uuid3(uuid.NAMESPACE_DNS, self.name)).split('-')[-1]
		if shell == 'auto':
			shell = re.sub(r'[^\w].*', '', path.basename(environ['SHELL']))
			return self.generate(shell, auto)
		self._inherit()
		if shell == 'fish':
			source = self._generateFish()
			if not auto:
				return source
			self._automateFish(source)
		elif shell == 'bash':
			source = self._generateBash()
			if not auto:
				return source
			self._automateBash(source)
		elif shell == 'zsh':
			source = self._generateZsh()
			if not auto:
				return source
			self._automateZsh(source)
		else:
			raise ValueError('Currently only bash, fish and zsh supported.')

	def load(self, dict_var):
		"""Load commands and options from a dict"""
		# integrity check
		if 'program' not in dict_var:
			raise CompletionsLoadError("No 'program' key found.")

		program = dict_var['program']
		if 'name' not in program:
			raise CompletionsLoadError("A program name should be given by 'program.name'")
		self.name = program['name']
		self.desc = program['desc']
		if sep in self.name:
			self.fullpath = path.realpath(self.name)
			self.name = path.basename(self.name)

		for key, val in program.get('options', {}).items():
			self.addOption(key, val)

		self.inherit = dict_var.get('inherit', True)
		commands = dict_var.get('commands', '')
		for key, val in commands.items():
			if not isinstance(val, dict):
				val = {'desc': val}
			options = val.get('options', {})
			self.addCommand(
				name    = key,
				desc    = val.get('desc') or '',
				options = options
			)

	def loadFile(self, compfile):
		"""Load commands and options from a configuration file"""
		config = Config(with_profile = False)
		config._load(compfile)
		self.load(config)

def main():
	"""Entry point of the script"""
	from pyparam import commands
	commands._desc = 'Shell completions for your program made easy.'
	commands._.shell          = 'auto'
	commands._.shell.desc     = [
		'The shell, one of bash, fish, zsh and auto.',
		'Shell will be detected from `os.environ["SHELL"]` if auto.',
	]
	commands._.auto           = False
	commands._.auto.desc      = [
		'Automatically write completions to destination file.',
		'Bash: `~/bash_completion.d/<name>.bash-completion`',
		'  Also try to source it in ~/.bash_completion',
		'Fish: `~/.config/fish/completions/<name>.fish`',
		'Zsh:  `~/.zsh-completions/_<name>`',
		'  `fpath+=~/.zsh-completions` is ensured to add before `compinit`'
	]
	commands._.a                  = commands._.auto
	commands._.s                  = commands._.shell
	commands.self                 = 'Generate completions for myself.'
	commands.self._hbald          = False
	commands.generate             = 'Generate completions from configuration files'
	commands.generate.config.desc = [ # pylint: disable=no-member
		'The configuration file. Scheme should be aligned following schema:',
		'```yaml',
		'program:',
		'    name: completions',
		'    desc: Shell completions for your program made easy.',
		'    path: /absolute/path/to/completions',
		'    inherit: true',
		'    options:',
		'        -s: The shell, one of bash, fish, zsh and auto.',
		'        --shell: The shell, one of bash, fish, zsh and auto.',
		'        -a: Automatically write completions to destination file.',
		'        --auto: Automatically write completions to destination file.',
		'commands:',
		'    self: Generate completions for myself.',
		'    generate:',
		'        desc: Generate completions from configuration files.',
		'        options:',
		'            -c: The configuration file to load.',
		'            --config: The configuration file to load.',
		'```',
		'Configuration file that is supported by `python-simpleconf` is supported.'
	]
	commands.generate.config.required = True       # pylint: disable=no-member
	commands.generate.c = commands.generate.config # pylint: disable=no-member
	command, options, goptions = commands._parse()

	auto = goptions['auto']
	if command == 'self':
		source = commands._complete(goptions['shell'], auto = auto)
		if not auto:
			print(source)
	else:
		completions = Completions()
		completions.loadFile(options['config'])
		source = completions.generate(goptions['shell'], auto = auto)
		if not auto:
			print(source)

if __name__ == '__main__':
	main()



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['completions']

package_data = \
{'': ['*']}

install_requires = \
['colorama', 'python-simpleconf>=0.1.4,<0.2.0']

entry_points = \
{'console_scripts': ['completions = completions:main']}

setup_kwargs = {
    'name': 'completions',
    'version': '0.0.2',
    'description': 'Shell completions made easy.',
    'long_description': "# completions\n\nShell completions for your program made easy.\n\n[![pypi][1]][2] [![tag][3]][4] [![travis][5]][6] [![codacy quality][7]][8] ![pyver][11]\n\nShell completions for your program made easy.\n\n## Installation\n```shell\npip install completions\n# install lastest version using poetry\ngit clone https://github.com/pwwang/completions\ncd completions\npoetry install\n```\n\n## Usage\n\n### Defining your completions\nYou may define your completions, basically commands and options, by following schema (showed in `yaml`, but can be any format supported by [`python-simpleconf`][12]:\n`example.yaml`\n```yaml\nprogram:\n    # your program, or path to your program\n    name: completions-example\n    desc: Shell completions for your program made easy.\n    # whether global options should be inherited by commands\n    inherit: true\n    # options or global options if you have commands\n    options:\n        -s: The shell, one of bash, fish, zsh and auto.\n        --shell: The shell, one of bash, fish, zsh and auto.\n        -a: Automatically write completions to destination file.\n        --auto: Automatically write completions to destination file.\ncommands:\n    # No other options for command, give the description\n    self: Generate completions for myself.\n    generate:\n        desc: Generate completions from configuration files.\n        options:\n            -c: The configuration file to load.\n            --config: The configuration file to load.\n```\n\nHow it looks like in `fish`:\n![command][13]\n![option][14]\n\n### Generating completion scripts\n- Bash\n    ```shell\n    > completions generate --shell bash \\\n        --config example.yaml > ~/bash_completion.d/completions.bash-completion\n    ```\n    You may need to `source` it in your `.bashrc` and restart your shell for the changes to take effect.\n- Fish\n    ```shell\n    > completions generate --shell fish \\\n        --config example.yaml > ~/.config/fish/completions/completions.fish\n    ```\n    You may need to restart your shell for the changes to take effect.\n- Zsh\n    ```shell\n    > completions generate --shell zsh \\\n        --config example.yaml > ~/.zsh-completions/_completions\n    ```\n    Make sure `fpath+=~/.zsh-completions` is put before `compinit` in you `.zshrc`\n\n### Saving completions scripts automatically\n- Bash\n    ```shell\n    > completions generate --shell bash --config example.yaml --auto\n    ```\n\n- Fish\n    ```shell\n    > completions generate --shell fish --config example.yaml --auto\n    ```\n\n- Zsh\n    ```shell\n    > completions generate --shell zsh --config example.yaml --auto\n    ```\n\n### Python API\n```python\nfrom completions import Completions\ncompletions = Completions(\n    # if not given, will be read from sys.argv[0]\n    name    = 'completions',\n    # Add global options to commands\n    inherit = True,\n    desc    = 'Shell completions for your program made easy.')\ncompletions.addOption(\n    ['-s', '--shell'],\n    'The shell, one of bash, fish, zsh and auto.')\ncompletions.addOption(\n    ['-a', '--auto'],\n    'Automatically write completions to destination file.')\ncompletions.addCommand(\n    'self', 'Generate completions for myself.')\ncompletions.addCommand(\n    'generate', 'Generate completions from configuration files.')\ncompletions.command('generate').addOption(\n    ['-c', '--config'], 'The configuration file to load.')\ncompletions.generate(shell = 'fish', auto = False)\n```\n\n[1]: https://img.shields.io/pypi/v/completions.svg?style=flat-square\n[2]: https://pypi.org/project/completions/\n[3]: https://img.shields.io/github/tag/pwwang/completions.svg?style=flat-square\n[4]: https://github.com/pwwang/completions\n[5]: https://img.shields.io/travis/pwwang/completions.svg?style=flat-square\n[6]: https://travis-ci.org/pwwang/completions\n[7]: https://img.shields.io/codacy/grade/completions.svg?style=flat-square\n[8]: https://app.codacy.com/project/pwwang/completions/dashboard\n[11]: https://img.shields.io/pypi/pyversions/completions.svg?style=flat-square\n[12]: https://github.com/pwwang/simpleconf\n[13]: https://raw.githubusercontent.com/pwwang/completions/master/examples/command.png\n[14]: https://raw.githubusercontent.com/pwwang/completions/master/examples/option.png\n",
    'author': 'pwwang',
    'author_email': 'pwwang@pwwang.com',
    'url': 'https://github.com/pwwang/completions',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.4,<4.0',
}


setup(**setup_kwargs)
